@@ -12,9 +12,9 @@ use url::Url;
 
 use ops;
 use sources::git;
-use sources::{PathSource, GitSource, RegistrySource, CRATES_IO};
+use sources::{GitSource, PathSource, RegistrySource, CRATES_IO};
 use sources::DirectorySource;
-use util::{Config, CargoResult, ToUrl};
+use util::{CargoResult, Config, ToUrl};
 
 /// Unique identifier for a source of packages.
 #[derive(Clone, Eq, Debug)]
@@ -36,7 +36,7 @@ struct SourceIdInner {
     name: Option<String>,
 }
 
-/// The possible kinds of code source. Along with a URL, this fully defines the
+/// The possible kinds of code source. Along with SourceIdInner this fully defines the
 /// source
 #[derive(Debug, Clone, PartialEq, Eq, PartialOrd, Ord, Hash)]
 enum Kind {
@@ -70,9 +70,9 @@ impl SourceId {
     fn new(kind: Kind, url: Url) -> CargoResult<SourceId> {
         let source_id = SourceId {
             inner: Arc::new(SourceIdInner {
-                kind: kind,
+                kind,
                 canonical_url: git::canonicalize_url(&url)?,
-                url: url,
+                url,
                 precise: None,
                 name: None,
             }),
@@ -93,7 +93,9 @@ impl SourceId {
     pub fn from_url(string: &str) -> CargoResult<SourceId> {
         let mut parts = string.splitn(2, '+');
         let kind = parts.next().unwrap();
-        let url = parts.next().ok_or_else(|| format_err!("invalid source `{}`", string))?;
+        let url = parts
+            .next()
+            .ok_or_else(|| format_err!("invalid source `{}`", string))?;
 
         match kind {
             "git" => {
@@ -102,8 +104,7 @@ impl SourceId {
                 for (k, v) in url.query_pairs() {
                     match &k[..] {
                         // map older 'ref' to branch
-                        "branch" |
-                        "ref" => reference = GitReference::Branch(v.into_owned()),
+                        "branch" | "ref" => reference = GitReference::Branch(v.into_owned()),
 
                         "rev" => reference = GitReference::Rev(v.into_owned()),
                         "tag" => reference = GitReference::Tag(v.into_owned()),
@@ -114,23 +115,24 @@ impl SourceId {
                 url.set_fragment(None);
                 url.set_query(None);
                 Ok(SourceId::for_git(&url, reference)?.with_precise(precise))
-            },
+            }
             "registry" => {
                 let url = url.to_url()?;
-                Ok(SourceId::new(Kind::Registry, url)?
-                            .with_precise(Some("locked".to_string())))
+                Ok(SourceId::new(Kind::Registry, url)?.with_precise(Some("locked".to_string())))
             }
             "path" => {
                 let url = url.to_url()?;
                 SourceId::new(Kind::Path, url)
             }
-            kind => Err(format_err!("unsupported source protocol: {}", kind))
+            kind => Err(format_err!("unsupported source protocol: {}", kind)),
         }
     }
 
     /// A view of the `SourceId` that can be `Display`ed as a URL
     pub fn to_url(&self) -> SourceIdToUrl {
-        SourceIdToUrl { inner: &*self.inner }
+        SourceIdToUrl {
+            inner: &*self.inner,
+        }
     }
 
     /// Create a SourceId from a filesystem path.
@@ -168,21 +170,25 @@ impl SourceId {
     /// This is the main cargo registry by default, but it can be overridden in
     /// a `.cargo/config`.
     pub fn crates_io(config: &Config) -> CargoResult<SourceId> {
-        let cfg = ops::registry_configuration(config, None)?;
-        let url = if let Some(ref index) = cfg.index {
-            static WARNED: AtomicBool = ATOMIC_BOOL_INIT;
-            if !WARNED.swap(true, SeqCst) {
-                config.shell().warn("custom registry support via \
-                                     the `registry.index` configuration is \
-                                     being removed, this functionality \
-                                     will not work in the future")?;
-            }
-            &index[..]
-        } else {
-            CRATES_IO
-        };
-        let url = url.to_url()?;
-        SourceId::for_registry(&url)
+        config.crates_io_source_id(|| {
+            let cfg = ops::registry_configuration(config, None)?;
+            let url = if let Some(ref index) = cfg.index {
+                static WARNED: AtomicBool = ATOMIC_BOOL_INIT;
+                if !WARNED.swap(true, SeqCst) {
+                    config.shell().warn(
+                        "custom registry support via \
+                         the `registry.index` configuration is \
+                         being removed, this functionality \
+                         will not work in the future",
+                    )?;
+                }
+                &index[..]
+            } else {
+                CRATES_IO
+            };
+            let url = url.to_url()?;
+            SourceId::for_registry(&url)
+        })
     }
 
     pub fn alt_registry(config: &Config, key: &str) -> CargoResult<SourceId> {
@@ -191,7 +197,7 @@ impl SourceId {
             inner: Arc::new(SourceIdInner {
                 kind: Kind::Registry,
                 canonical_url: git::canonicalize_url(&url)?,
-                url: url,
+                url,
                 precise: None,
                 name: Some(key.to_string()),
             }),
@@ -216,7 +222,7 @@ impl SourceId {
     pub fn is_registry(&self) -> bool {
         match self.inner.kind {
             Kind::Registry | Kind::LocalRegistry => true,
-            _                                    => false,
+            _ => false,
         }
     }
 
@@ -282,7 +288,7 @@ impl SourceId {
             inner: Arc::new(SourceIdInner {
                 precise: v,
                 ..(*self.inner).clone()
-            })
+            }),
         }
     }
 
@@ -301,10 +307,15 @@ impl SourceId {
     /// same hash in different locations.
     pub fn stable_hash<S: hash::Hasher>(&self, workspace: &Path, into: &mut S) {
         if self.is_path() {
-            if let Ok(p) = self.inner.url.to_file_path().unwrap().strip_prefix(workspace) {
+            if let Ok(p) = self.inner
+                .url
+                .to_file_path()
+                .unwrap()
+                .strip_prefix(workspace)
+            {
                 self.inner.kind.hash(into);
                 p.to_str().unwrap().hash(into);
-                return
+                return;
             }
         }
         self.hash(into)
@@ -331,7 +342,8 @@ impl Ord for SourceId {
 
 impl ser::Serialize for SourceId {
     fn serialize<S>(&self, s: S) -> Result<S::Ok, S::Error>
-        where S: ser::Serializer,
+    where
+        S: ser::Serializer,
     {
         if self.is_path() {
             None::<String>.serialize(s)
@@ -343,7 +355,8 @@ impl ser::Serialize for SourceId {
 
 impl<'de> de::Deserialize<'de> for SourceId {
     fn deserialize<D>(d: D) -> Result<SourceId, D::Error>
-        where D: de::Deserializer<'de>,
+    where
+        D: de::Deserializer<'de>,
     {
         let string = String::deserialize(d)?;
         SourceId::from_url(&string).map_err(de::Error::custom)
@@ -353,11 +366,17 @@ impl<'de> de::Deserialize<'de> for SourceId {
 impl fmt::Display for SourceId {
     fn fmt(&self, f: &mut Formatter) -> fmt::Result {
         match *self.inner {
-            SourceIdInner { kind: Kind::Path, ref url, .. } => {
-                fmt::Display::fmt(url, f)
-            }
-            SourceIdInner { kind: Kind::Git(ref reference), ref url,
-                            ref precise, .. } => {
+            SourceIdInner {
+                kind: Kind::Path,
+                ref url,
+                ..
+            } => fmt::Display::fmt(url, f),
+            SourceIdInner {
+                kind: Kind::Git(ref reference),
+                ref url,
+                ref precise,
+                ..
+            } => {
                 write!(f, "{}", url)?;
                 if let Some(pretty) = reference.pretty_ref() {
                     write!(f, "?{}", pretty)?;
@@ -369,13 +388,21 @@ impl fmt::Display for SourceId {
                 }
                 Ok(())
             }
-            SourceIdInner { kind: Kind::Registry, ref url, .. } |
-            SourceIdInner { kind: Kind::LocalRegistry, ref url, .. } => {
-                write!(f, "registry `{}`", url)
-            }
-            SourceIdInner { kind: Kind::Directory, ref url, .. } => {
-                write!(f, "dir {}", url)
+            SourceIdInner {
+                kind: Kind::Registry,
+                ref url,
+                ..
             }
+            | SourceIdInner {
+                kind: Kind::LocalRegistry,
+                ref url,
+                ..
+            } => write!(f, "registry `{}`", url),
+            SourceIdInner {
+                kind: Kind::Directory,
+                ref url,
+                ..
+            } => write!(f, "dir {}", url),
         }
     }
 }
@@ -436,15 +463,17 @@ impl Hash for SourceId {
     fn hash<S: hash::Hasher>(&self, into: &mut S) {
         self.inner.kind.hash(into);
         match *self.inner {
-            SourceIdInner { kind: Kind::Git(..), ref canonical_url, .. } => {
-                canonical_url.as_str().hash(into)
-            }
+            SourceIdInner {
+                kind: Kind::Git(..),
+                ref canonical_url,
+                ..
+            } => canonical_url.as_str().hash(into),
             _ => self.inner.url.as_str().hash(into),
         }
     }
 }
 
-/// A `Display`able view into a SourceId that will write it as a url
+/// A `Display`able view into a `SourceId` that will write it as a url
 pub struct SourceIdToUrl<'a> {
     inner: &'a SourceIdInner,
 }
@@ -452,11 +481,16 @@ pub struct SourceIdToUrl<'a> {
 impl<'a> fmt::Display for SourceIdToUrl<'a> {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         match *self.inner {
-            SourceIdInner { kind: Kind::Path, ref url, .. } => {
-                write!(f, "path+{}", url)
-            }
             SourceIdInner {
-                kind: Kind::Git(ref reference), ref url, ref precise, ..
+                kind: Kind::Path,
+                ref url,
+                ..
+            } => write!(f, "path+{}", url),
+            SourceIdInner {
+                kind: Kind::Git(ref reference),
+                ref url,
+                ref precise,
+                ..
             } => {
                 write!(f, "git+{}", url)?;
                 if let Some(pretty) = reference.pretty_ref() {
@@ -467,15 +501,21 @@ impl<'a> fmt::Display for SourceIdToUrl<'a> {
                 }
                 Ok(())
             }
-            SourceIdInner { kind: Kind::Registry, ref url, .. } => {
-                write!(f, "registry+{}", url)
-            }
-            SourceIdInner { kind: Kind::LocalRegistry, ref url, .. } => {
-                write!(f, "local-registry+{}", url)
-            }
-            SourceIdInner { kind: Kind::Directory, ref url, .. } => {
-                write!(f, "directory+{}", url)
-            }
+            SourceIdInner {
+                kind: Kind::Registry,
+                ref url,
+                ..
+            } => write!(f, "registry+{}", url),
+            SourceIdInner {
+                kind: Kind::LocalRegistry,
+                ref url,
+                ..
+            } => write!(f, "local-registry+{}", url),
+            SourceIdInner {
+                kind: Kind::Directory,
+                ref url,
+                ..
+            } => write!(f, "directory+{}", url),
         }
     }
 }
@@ -508,7 +548,7 @@ impl<'a> fmt::Display for PrettyRef<'a> {
 
 #[cfg(test)]
 mod tests {
-    use super::{SourceId, Kind, GitReference};
+    use super::{GitReference, Kind, SourceId};
     use util::ToUrl;
 
     #[test]
@@ -524,6 +564,6 @@ mod tests {
 
         let foo = Kind::Git(GitReference::Branch("foo".to_string()));
         let s3 = SourceId::new(foo, loc).unwrap();
-        assert!(s1 != s3);
+        assert_ne!(s1, s3);
     }
 }
