@@ -1,17 +1,18 @@
-#![deny(unused)]
 #![cfg_attr(test, deny(warnings))]
-#![recursion_limit="128"]
+// Currently, Cargo does not use clippy for its source code.
+// But if someone runs it they should know that
+// @alexcrichton disagree with clippy on some style things
+#![cfg_attr(feature = "cargo-clippy", allow(explicit_iter_loop))]
 
-#[macro_use] extern crate failure;
-#[macro_use] extern crate log;
-#[macro_use] extern crate scoped_tls;
-#[macro_use] extern crate serde_derive;
-#[macro_use] extern crate serde_json;
 extern crate atty;
+extern crate clap;
+#[cfg(target_os = "macos")]
+extern crate core_foundation;
 extern crate crates_io as registry;
 extern crate crossbeam;
 extern crate curl;
-extern crate docopt;
+#[macro_use]
+extern crate failure;
 extern crate filetime;
 extern crate flate2;
 extern crate fs2;
@@ -21,28 +22,33 @@ extern crate hex;
 extern crate home;
 extern crate ignore;
 extern crate jobserver;
+#[macro_use]
+extern crate lazy_static;
 extern crate lazycell;
 extern crate libc;
 extern crate libgit2_sys;
+#[macro_use]
+extern crate log;
 extern crate num_cpus;
 extern crate same_file;
 extern crate semver;
+#[macro_use]
 extern crate serde;
+#[macro_use]
+extern crate serde_derive;
 extern crate serde_ignored;
+#[macro_use]
+extern crate serde_json;
 extern crate shell_escape;
 extern crate tar;
-extern crate tempdir;
+extern crate tempfile;
 extern crate termcolor;
 extern crate toml;
 extern crate url;
-#[cfg(target_os = "macos")]
-extern crate core_foundation;
 
 use std::fmt;
 
-use serde::Deserialize;
 use serde::ser;
-use docopt::Docopt;
 use failure::Error;
 
 use core::Shell;
@@ -51,7 +57,7 @@ use core::shell::Verbosity::Verbose;
 pub use util::{CargoError, CargoResult, CliError, CliResult, Config};
 pub use util::errors::Internal;
 
-pub const CARGO_ENV: &'static str = "CARGO";
+pub const CARGO_ENV: &str = "CARGO";
 
 pub mod core;
 pub mod ops;
@@ -72,9 +78,9 @@ pub struct CfgInfo {
 }
 
 pub struct VersionInfo {
-    pub major: String,
-    pub minor: String,
-    pub patch: String,
+    pub major: u8,
+    pub minor: u8,
+    pub patch: u8,
     pub pre_release: Option<String>,
     // Information that's only available when we were built with
     // configure/make, rather than cargo itself.
@@ -83,8 +89,7 @@ pub struct VersionInfo {
 
 impl fmt::Display for VersionInfo {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        write!(f, "cargo {}.{}.{}",
-               self.major, self.minor, self.patch)?;
+        write!(f, "cargo {}.{}.{}", self.major, self.minor, self.patch)?;
         if let Some(channel) = self.cfg_info.as_ref().map(|ci| &ci.release_channel) {
             if channel != "stable" {
                 write!(f, "-{}", channel)?;
@@ -95,34 +100,13 @@ impl fmt::Display for VersionInfo {
 
         if let Some(ref cfg) = self.cfg_info {
             if let Some(ref ci) = cfg.commit_info {
-                write!(f, " ({} {})",
-                       ci.short_commit_hash, ci.commit_date)?;
+                write!(f, " ({} {})", ci.short_commit_hash, ci.commit_date)?;
             }
         };
         Ok(())
     }
 }
 
-pub fn call_main_without_stdin<'de, Flags: Deserialize<'de>>(
-            exec: fn(Flags, &mut Config) -> CliResult,
-            config: &mut Config,
-            usage: &str,
-            args: &[String],
-            options_first: bool) -> CliResult
-{
-    let docopt = Docopt::new(usage).unwrap()
-        .options_first(options_first)
-        .argv(args.iter().map(|s| &s[..]))
-        .help(true);
-
-    let flags = docopt.deserialize().map_err(|e| {
-        let code = if e.fatal() {1} else {0};
-        CliError::new(e.into(), code)
-    })?;
-
-    exec(flags, config)
-}
-
 pub fn print_json<T: ser::Serialize>(obj: &T) {
     let encoded = serde_json::to_string(&obj).unwrap();
     println!("{}", encoded);
@@ -130,8 +114,17 @@ pub fn print_json<T: ser::Serialize>(obj: &T) {
 
 pub fn exit_with_error(err: CliError, shell: &mut Shell) -> ! {
     debug!("exit_with_error; err={:?}", err);
+    if let Some(ref err) = err.error {
+        if let Some(clap_err) = err.downcast_ref::<clap::Error>() {
+            clap_err.exit()
+        }
+    }
 
-    let CliError { error, exit_code, unknown } = err;
+    let CliError {
+        error,
+        exit_code,
+        unknown,
+    } = err;
     // exit_code == 0 is non-fatal error, e.g. docopt version info
     let fatal = exit_code != 0;
 
@@ -147,8 +140,11 @@ pub fn exit_with_error(err: CliError, shell: &mut Shell) -> ! {
         }
 
         if !handle_cause(&error, shell) || hide {
-            drop(writeln!(shell.err(), "\nTo learn more, run the command again \
-                                        with --verbose."));
+            drop(writeln!(
+                shell.err(),
+                "\nTo learn more, run the command again \
+                 with --verbose."
+            ));
         }
     }
 
@@ -192,43 +188,54 @@ fn handle_cause(cargo_err: &Error, shell: &mut Shell) -> bool {
 }
 
 pub fn version() -> VersionInfo {
-    macro_rules! env_str {
-        ($name:expr) => { env!($name).to_string() }
-    }
     macro_rules! option_env_str {
         ($name:expr) => { option_env!($name).map(|s| s.to_string()) }
     }
+
+    // So this is pretty horrible...
+    // There are two versions at play here:
+    //   - version of cargo-the-binary, which you see when you type `cargo --version`
+    //   - version of cargo-the-library, which you download from crates.io for use
+    //     in your projects.
+    //
+    // We want to make the `binary` version the same as the corresponding Rust/rustc release.
+    // At the same time, we want to keep the library version at `0.x`, because Cargo as
+    // a library is (and probably will always be) unstable.
+    //
+    // Historically, Cargo used the same version number for both the binary and the library.
+    // Specifically, rustc 1.x.z was paired with cargo 0.x+1.w.
+    // We continue to use this scheme for the library, but transform it to 1.x.w for the purposes
+    // of `cargo --version`.
+    let major = 1;
+    let minor = env!("CARGO_PKG_VERSION_MINOR").parse::<u8>().unwrap() - 1;
+    let patch = env!("CARGO_PKG_VERSION_PATCH").parse::<u8>().unwrap();
+
     match option_env!("CFG_RELEASE_CHANNEL") {
         // We have environment variables set up from configure/make.
         Some(_) => {
-            let commit_info =
-                option_env!("CFG_COMMIT_HASH").map(|s| {
-                    CommitInfo {
-                        commit_hash: s.to_string(),
-                        short_commit_hash: option_env_str!("CFG_SHORT_COMMIT_HASH").unwrap(),
-                        commit_date: option_env_str!("CFG_COMMIT_DATE").unwrap(),
-                    }
-                });
+            let commit_info = option_env!("CFG_COMMIT_HASH").map(|s| CommitInfo {
+                commit_hash: s.to_string(),
+                short_commit_hash: option_env_str!("CFG_SHORT_COMMIT_HASH").unwrap(),
+                commit_date: option_env_str!("CFG_COMMIT_DATE").unwrap(),
+            });
             VersionInfo {
-                major: env_str!("CARGO_PKG_VERSION_MAJOR"),
-                minor: env_str!("CARGO_PKG_VERSION_MINOR"),
-                patch: env_str!("CARGO_PKG_VERSION_PATCH"),
+                major,
+                minor,
+                patch,
                 pre_release: option_env_str!("CARGO_PKG_VERSION_PRE"),
                 cfg_info: Some(CfgInfo {
                     release_channel: option_env_str!("CFG_RELEASE_CHANNEL").unwrap(),
-                    commit_info: commit_info,
+                    commit_info,
                 }),
             }
-        },
-        // We are being compiled by Cargo itself.
-        None => {
-            VersionInfo {
-                major: env_str!("CARGO_PKG_VERSION_MAJOR"),
-                minor: env_str!("CARGO_PKG_VERSION_MINOR"),
-                patch: env_str!("CARGO_PKG_VERSION_PATCH"),
-                pre_release: option_env_str!("CARGO_PKG_VERSION_PRE"),
-                cfg_info: None,
-            }
         }
+        // We are being compiled by Cargo itself.
+        None => VersionInfo {
+            major,
+            minor,
+            patch,
+            pre_release: option_env_str!("CARGO_PKG_VERSION_PRE"),
+            cfg_info: None,
+        },
     }
 }
