@@ -5,15 +5,17 @@ use std::path::{self, Path};
 use std::sync::Arc;
 
 use flate2::read::GzDecoder;
-use flate2::{GzBuilder, Compression};
+use flate2::{Compression, GzBuilder};
 use git2;
-use tar::{Archive, Builder, Header, EntryType};
+use tar::{Archive, Builder, EntryType, Header};
 
-use core::{Package, Workspace, Source, SourceId};
+use core::{Package, Source, SourceId, Workspace};
+use core::compiler::{BuildConfig, CompileMode, DefaultExecutor};
 use sources::PathSource;
 use util::{self, internal, Config, FileLock};
+use util::paths;
 use util::errors::{CargoResult, CargoResultExt};
-use ops::{self, DefaultExecutor};
+use ops;
 
 pub struct PackageOpts<'cfg> {
     pub config: &'cfg Config,
@@ -22,18 +24,16 @@ pub struct PackageOpts<'cfg> {
     pub allow_dirty: bool,
     pub verify: bool,
     pub jobs: Option<u32>,
-    pub target: Option<&'cfg str>,
+    pub target: Option<String>,
     pub registry: Option<String>,
 }
 
-pub fn package(ws: &Workspace,
-               opts: &PackageOpts) -> CargoResult<Option<FileLock>> {
+pub fn package(ws: &Workspace, opts: &PackageOpts) -> CargoResult<Option<FileLock>> {
+    ops::resolve_ws(ws)?;
     let pkg = ws.current()?;
     let config = ws.config();
 
-    let mut src = PathSource::new(pkg.root(),
-                                  pkg.package_id().source_id(),
-                                  config);
+    let mut src = PathSource::new(pkg.root(), pkg.package_id().source_id(), config);
     src.update()?;
 
     if opts.check_metadata {
@@ -44,14 +44,18 @@ pub fn package(ws: &Workspace,
 
     if opts.list {
         let root = pkg.root();
-        let mut list: Vec<_> = src.list_files(pkg)?.iter().map(|file| {
-            util::without_prefix(file, root).unwrap().to_path_buf()
-        }).collect();
+        let mut list: Vec<_> = src.list_files(pkg)?
+            .iter()
+            .map(|file| util::without_prefix(file, root).unwrap().to_path_buf())
+            .collect();
+        if include_lockfile(pkg) {
+            list.push("Cargo.lock".into());
+        }
         list.sort();
         for file in list.iter() {
             println!("{}", file.display());
         }
-        return Ok(None)
+        return Ok(None);
     }
 
     if !opts.allow_dirty {
@@ -69,28 +73,30 @@ pub fn package(ws: &Workspace,
     // location if it actually passes all our tests. Any previously existing
     // tarball can be assumed as corrupt or invalid, so we just blow it away if
     // it exists.
-    config.shell().status("Packaging", pkg.package_id().to_string())?;
+    config
+        .shell()
+        .status("Packaging", pkg.package_id().to_string())?;
     dst.file().set_len(0)?;
-    tar(ws, &src, dst.file(), &filename).chain_err(|| {
-        format_err!("failed to prepare local package for uploading")
-    })?;
+    tar(ws, &src, dst.file(), &filename)
+        .chain_err(|| format_err!("failed to prepare local package for uploading"))?;
     if opts.verify {
         dst.seek(SeekFrom::Start(0))?;
-        run_verify(ws, &dst, opts).chain_err(|| {
-            "failed to verify package tarball"
-        })?
+        run_verify(ws, &dst, opts).chain_err(|| "failed to verify package tarball")?
     }
     dst.seek(SeekFrom::Start(0))?;
     {
         let src_path = dst.path();
         let dst_path = dst.parent().join(&filename);
-        fs::rename(&src_path, &dst_path).chain_err(|| {
-            "failed to move temporary tarball into final location"
-        })?;
+        fs::rename(&src_path, &dst_path)
+            .chain_err(|| "failed to move temporary tarball into final location")?;
     }
     Ok(Some(dst))
 }
 
+fn include_lockfile(pkg: &Package) -> bool {
+    pkg.manifest().publish_lockfile() && pkg.targets().iter().any(|t| t.is_example() || t.is_bin())
+}
+
 // check that the package has some piece of metadata that a human can
 // use to tell what the package is about.
 fn check_metadata(pkg: &Package, config: &Config) -> CargoResult<()> {
@@ -107,7 +113,11 @@ fn check_metadata(pkg: &Package, config: &Config) -> CargoResult<()> {
             )*
         }}
     }
-    lacking!(description, license || license_file, documentation || homepage || repository);
+    lacking!(
+        description,
+        license || license_file,
+        documentation || homepage || repository
+    );
 
     if !missing.is_empty() {
         let mut things = missing[..missing.len() - 1].join(", ");
@@ -118,10 +128,11 @@ fn check_metadata(pkg: &Package, config: &Config) -> CargoResult<()> {
         }
         things.push_str(missing.last().unwrap());
 
-        config.shell().warn(
-            &format!("manifest has no {things}.\n\
-                    See http://doc.crates.io/manifest.html#package-metadata for more info.",
-                    things = things))?
+        config.shell().warn(&format!(
+            "manifest has no {things}.\n\
+             See http://doc.crates.io/manifest.html#package-metadata for more info.",
+            things = things
+        ))?
     }
     Ok(())
 }
@@ -130,9 +141,12 @@ fn check_metadata(pkg: &Package, config: &Config) -> CargoResult<()> {
 fn verify_dependencies(pkg: &Package) -> CargoResult<()> {
     for dep in pkg.dependencies() {
         if dep.source_id().is_path() && !dep.specified_req() {
-            bail!("all path dependencies must have a version specified \
-                    when packaging.\ndependency `{}` does not specify \
-                    a version.", dep.name())
+            bail!(
+                "all path dependencies must have a version specified \
+                 when packaging.\ndependency `{}` does not specify \
+                 a version.",
+                dep.name()
+            )
         }
     }
     Ok(())
@@ -141,14 +155,16 @@ fn verify_dependencies(pkg: &Package) -> CargoResult<()> {
 fn check_not_dirty(p: &Package, src: &PathSource) -> CargoResult<()> {
     if let Ok(repo) = git2::Repository::discover(p.root()) {
         if let Some(workdir) = repo.workdir() {
-            debug!("found a git repo at {:?}, checking if index present",
-                   workdir);
+            debug!(
+                "found a git repo at {:?}, checking if index present",
+                workdir
+            );
             let path = p.manifest_path();
             let path = path.strip_prefix(workdir).unwrap_or(path);
             if let Ok(status) = repo.status_file(path) {
-                if (status & git2::STATUS_IGNORED).is_empty() {
+                if (status & git2::Status::IGNORED).is_empty() {
                     debug!("Cargo.toml found in repo, checking if dirty");
-                    return git(p, src, &repo)
+                    return git(p, src, &repo);
                 }
             }
         }
@@ -158,39 +174,45 @@ fn check_not_dirty(p: &Package, src: &PathSource) -> CargoResult<()> {
     // have to assume that it's clean.
     return Ok(());
 
-    fn git(p: &Package,
-           src: &PathSource,
-           repo: &git2::Repository) -> CargoResult<()> {
+    fn git(p: &Package, src: &PathSource, repo: &git2::Repository) -> CargoResult<()> {
         let workdir = repo.workdir().unwrap();
-        let dirty = src.list_files(p)?.iter().filter(|file| {
-            let relative = file.strip_prefix(workdir).unwrap();
-            if let Ok(status) = repo.status_file(relative) {
-                status != git2::STATUS_CURRENT
-            } else {
-                false
-            }
-        }).map(|path| {
-            path.strip_prefix(p.root()).unwrap_or(path).display().to_string()
-        }).collect::<Vec<_>>();
+        let dirty = src.list_files(p)?
+            .iter()
+            .filter(|file| {
+                let relative = file.strip_prefix(workdir).unwrap();
+                if let Ok(status) = repo.status_file(relative) {
+                    status != git2::Status::CURRENT
+                } else {
+                    false
+                }
+            })
+            .map(|path| {
+                path.strip_prefix(p.root())
+                    .unwrap_or(path)
+                    .display()
+                    .to_string()
+            })
+            .collect::<Vec<_>>();
         if dirty.is_empty() {
             Ok(())
         } else {
-            bail!("{} files in the working directory contain changes that were \
-                   not yet committed into git:\n\n{}\n\n\
-                   to proceed despite this, pass the `--allow-dirty` flag",
-                  dirty.len(), dirty.join("\n"))
+            bail!(
+                "{} files in the working directory contain changes that were \
+                 not yet committed into git:\n\n{}\n\n\
+                 to proceed despite this, pass the `--allow-dirty` flag",
+                dirty.len(),
+                dirty.join("\n")
+            )
         }
     }
 }
 
-fn tar(ws: &Workspace,
-       src: &PathSource,
-       dst: &File,
-       filename: &str) -> CargoResult<()> {
+fn tar(ws: &Workspace, src: &PathSource, dst: &File, filename: &str) -> CargoResult<()> {
     // Prepare the encoder and its header
     let filename = Path::new(filename);
-    let encoder = GzBuilder::new().filename(util::path2bytes(filename)?)
-                                  .write(dst, Compression::best());
+    let encoder = GzBuilder::new()
+        .filename(util::path2bytes(filename)?)
+        .write(dst, Compression::best());
 
     // Put all package files into a compressed archive
     let mut ar = Builder::new(encoder);
@@ -201,14 +223,18 @@ fn tar(ws: &Workspace,
         let relative = util::without_prefix(file, root).unwrap();
         check_filename(relative)?;
         let relative = relative.to_str().ok_or_else(|| {
-            format_err!("non-utf8 path in source directory: {}",
-                        relative.display())
+            format_err!("non-utf8 path in source directory: {}", relative.display())
         })?;
-        config.shell().verbose(|shell| {
-            shell.status("Archiving", &relative)
-        })?;
-        let path = format!("{}-{}{}{}", pkg.name(), pkg.version(),
-                           path::MAIN_SEPARATOR, relative);
+        config
+            .shell()
+            .verbose(|shell| shell.status("Archiving", &relative))?;
+        let path = format!(
+            "{}-{}{}{}",
+            pkg.name(),
+            pkg.version(),
+            path::MAIN_SEPARATOR,
+            relative
+        );
 
         // The tar::Builder type by default will build GNU archives, but
         // unfortunately we force it here to use UStar archives instead. The
@@ -229,24 +255,21 @@ fn tar(ws: &Workspace,
         // unpack the selectors 0.4.0 crate on crates.io. Either that or take a
         // look at rust-lang/cargo#2326
         let mut header = Header::new_ustar();
-        header.set_path(&path).chain_err(|| {
-            format!("failed to add to archive: `{}`", relative)
-        })?;
-        let mut file = File::open(file).chain_err(|| {
-            format!("failed to open for archiving: `{}`", file.display())
-        })?;
-        let metadata = file.metadata().chain_err(|| {
-            format!("could not learn metadata for: `{}`", relative)
-        })?;
+        header
+            .set_path(&path)
+            .chain_err(|| format!("failed to add to archive: `{}`", relative))?;
+        let mut file = File::open(file)
+            .chain_err(|| format!("failed to open for archiving: `{}`", file.display()))?;
+        let metadata = file.metadata()
+            .chain_err(|| format!("could not learn metadata for: `{}`", relative))?;
         header.set_metadata(&metadata);
 
         if relative == "Cargo.toml" {
             let orig = Path::new(&path).with_file_name("Cargo.toml.orig");
             header.set_path(&orig)?;
             header.set_cksum();
-            ar.append(&header, &mut file).chain_err(|| {
-                internal(format!("could not archive source file `{}`", relative))
-            })?;
+            ar.append(&header, &mut file)
+                .chain_err(|| internal(format!("could not archive source file `{}`", relative)))?;
 
             let mut header = Header::new_ustar();
             let toml = pkg.to_registry_toml(ws.config())?;
@@ -255,16 +278,33 @@ fn tar(ws: &Workspace,
             header.set_mode(0o644);
             header.set_size(toml.len() as u64);
             header.set_cksum();
-            ar.append(&header, toml.as_bytes()).chain_err(|| {
-                internal(format!("could not archive source file `{}`", relative))
-            })?;
+            ar.append(&header, toml.as_bytes())
+                .chain_err(|| internal(format!("could not archive source file `{}`", relative)))?;
         } else {
             header.set_cksum();
-            ar.append(&header, &mut file).chain_err(|| {
-                internal(format!("could not archive source file `{}`", relative))
-            })?;
+            ar.append(&header, &mut file)
+                .chain_err(|| internal(format!("could not archive source file `{}`", relative)))?;
         }
     }
+
+    if include_lockfile(pkg) {
+        let toml = paths::read(&ws.root().join("Cargo.lock"))?;
+        let path = format!(
+            "{}-{}{}Cargo.lock",
+            pkg.name(),
+            pkg.version(),
+            path::MAIN_SEPARATOR
+        );
+        let mut header = Header::new_ustar();
+        header.set_path(&path)?;
+        header.set_entry_type(EntryType::file());
+        header.set_mode(0o644);
+        header.set_size(toml.len() as u64);
+        header.set_cksum();
+        ar.append(&header, toml.as_bytes())
+            .chain_err(|| internal("could not archive source file `Cargo.lock`"))?;
+    }
+
     let encoder = ar.into_inner()?;
     encoder.finish()?;
     Ok(())
@@ -277,9 +317,10 @@ fn run_verify(ws: &Workspace, tar: &FileLock, opts: &PackageOpts) -> CargoResult
     config.shell().status("Verifying", pkg)?;
 
     let f = GzDecoder::new(tar.file());
-    let dst = tar.parent().join(&format!("{}-{}", pkg.name(), pkg.version()));
-    if fs::metadata(&dst).is_ok() {
-        fs::remove_dir_all(&dst)?;
+    let dst = tar.parent()
+        .join(&format!("{}-{}", pkg.name(), pkg.version()));
+    if dst.exists() {
+        paths::remove_dir_all(&dst)?;
     }
     let mut archive = Archive::new(f);
     archive.unpack(dst.parent().unwrap())?;
@@ -289,23 +330,41 @@ fn run_verify(ws: &Workspace, tar: &FileLock, opts: &PackageOpts) -> CargoResult
     let id = SourceId::for_path(&dst)?;
     let mut src = PathSource::new(&dst, &id, ws.config());
     let new_pkg = src.root_package()?;
+    let pkg_fingerprint = src.last_modified_file(&new_pkg)?;
     let ws = Workspace::ephemeral(new_pkg, config, None, true)?;
 
-    ops::compile_ws(&ws, None, &ops::CompileOptions {
-        config: config,
-        jobs: opts.jobs,
-        target: opts.target,
-        features: &[],
-        no_default_features: false,
-        all_features: false,
-        spec: ops::Packages::Packages(&[]),
-        filter: ops::CompileFilter::Default { required_features_filterable: true },
-        release: false,
-        message_format: ops::MessageFormat::Human,
-        mode: ops::CompileMode::Build,
-        target_rustdoc_args: None,
-        target_rustc_args: None,
-    }, Arc::new(DefaultExecutor))?;
+    ops::compile_ws(
+        &ws,
+        None,
+        &ops::CompileOptions {
+            config,
+            build_config: BuildConfig::new(config, opts.jobs, &opts.target, CompileMode::Build)?,
+            features: Vec::new(),
+            no_default_features: false,
+            all_features: false,
+            spec: ops::Packages::Packages(Vec::new()),
+            filter: ops::CompileFilter::Default {
+                required_features_filterable: true,
+            },
+            target_rustdoc_args: None,
+            target_rustc_args: None,
+            export_dir: None,
+        },
+        Arc::new(DefaultExecutor),
+    )?;
+
+    // Check that build.rs didn't modify any files in the src directory.
+    let ws_fingerprint = src.last_modified_file(ws.current()?)?;
+    if pkg_fingerprint != ws_fingerprint {
+        let (_, path) = ws_fingerprint;
+        bail!(
+            "Source directory was modified by build.rs during cargo publish. \
+             Build scripts should not modify anything outside of OUT_DIR. \
+             Modified file: {}\n\n\
+             To proceed despite this, pass the `--no-verify` flag.",
+            path.display()
+        )
+    }
 
     Ok(())
 }
@@ -323,15 +382,19 @@ fn check_filename(file: &Path) -> CargoResult<()> {
     };
     let name = match name.to_str() {
         Some(name) => name,
-        None => {
-            bail!("path does not have a unicode filename which may not unpack \
-                   on all platforms: {}", file.display())
-        }
+        None => bail!(
+            "path does not have a unicode filename which may not unpack \
+             on all platforms: {}",
+            file.display()
+        ),
     };
     let bad_chars = ['/', '\\', '<', '>', ':', '"', '|', '?', '*'];
     if let Some(c) = bad_chars.iter().find(|c| name.contains(**c)) {
-        bail!("cannot package a filename with a special character `{}`: {}",
-              c, file.display())
+        bail!(
+            "cannot package a filename with a special character `{}`: {}",
+            c,
+            file.display()
+        )
     }
     Ok(())
 }
