@@ -86,7 +86,7 @@ mod imp {
 
         let job = CreateJobObjectW(0 as *mut _, 0 as *const _);
         if job.is_null() {
-            return None
+            return None;
         }
         let job = Handle { inner: job };
 
@@ -96,14 +96,15 @@ mod imp {
         // our children will reside in the job once we spawn a process.
         let mut info: JOBOBJECT_EXTENDED_LIMIT_INFORMATION;
         info = mem::zeroed();
-        info.BasicLimitInformation.LimitFlags =
-            JOB_OBJECT_LIMIT_KILL_ON_JOB_CLOSE;
-        let r = SetInformationJobObject(job.inner,
-                        JobObjectExtendedLimitInformation,
-                        &mut info as *mut _ as LPVOID,
-                        mem::size_of_val(&info) as DWORD);
+        info.BasicLimitInformation.LimitFlags = JOB_OBJECT_LIMIT_KILL_ON_JOB_CLOSE;
+        let r = SetInformationJobObject(
+            job.inner,
+            JobObjectExtendedLimitInformation,
+            &mut info as *mut _ as LPVOID,
+            mem::size_of_val(&info) as DWORD,
+        );
         if r == 0 {
-            return None
+            return None;
         }
 
         // Assign our process to this job object, meaning that our children will
@@ -111,10 +112,10 @@ mod imp {
         let me = GetCurrentProcess();
         let r = AssignProcessToJobObject(job.inner, me);
         if r == 0 {
-            return None
+            return None;
         }
 
-        Some(Setup { job: job })
+        Some(Setup { job })
     }
 
     impl Drop for Setup {
@@ -140,13 +141,13 @@ mod imp {
                 let mut info: JOBOBJECT_EXTENDED_LIMIT_INFORMATION;
                 info = mem::zeroed();
                 let r = SetInformationJobObject(
-                            self.job.inner,
-                            JobObjectExtendedLimitInformation,
-                            &mut info as *mut _ as LPVOID,
-                            mem::size_of_val(&info) as DWORD);
+                    self.job.inner,
+                    JobObjectExtendedLimitInformation,
+                    &mut info as *mut _ as LPVOID,
+                    mem::size_of_val(&info) as DWORD,
+                );
                 if r == 0 {
-                    info!("failed to configure job object to defaults: {}",
-                          last_err());
+                    info!("failed to configure job object to defaults: {}", last_err());
                 }
             }
         }
@@ -162,14 +163,15 @@ mod imp {
 
             let mut jobs: Jobs = mem::zeroed();
             let r = QueryInformationJobObject(
-                            self.job.inner,
-                            JobObjectBasicProcessIdList,
-                            &mut jobs as *mut _ as LPVOID,
-                            mem::size_of_val(&jobs) as DWORD,
-                            0 as *mut _);
+                self.job.inner,
+                JobObjectBasicProcessIdList,
+                &mut jobs as *mut _ as LPVOID,
+                mem::size_of_val(&jobs) as DWORD,
+                0 as *mut _,
+            );
             if r == 0 {
                 info!("failed to query job object: {}", last_err());
-                return false
+                return false;
             }
 
             let mut killed = false;
@@ -177,46 +179,44 @@ mod imp {
             assert!(list.len() > 0);
             info!("found {} remaining processes", list.len() - 1);
 
-            let list = list.iter().filter(|&&id| {
-                // let's not kill ourselves
-                id as DWORD != GetCurrentProcessId()
-            }).filter_map(|&id| {
-                // Open the process with the necessary rights, and if this
-                // fails then we probably raced with the process exiting so we
-                // ignore the problem.
-                let flags = PROCESS_QUERY_INFORMATION |
-                            PROCESS_TERMINATE |
-                            SYNCHRONIZE;
-                let p = OpenProcess(flags, FALSE, id as DWORD);
-                if p.is_null() {
-                    None
-                } else {
-                    Some(Handle { inner: p })
-                }
-            }).filter(|p| {
-                // Test if this process was actually in the job object or not.
-                // If it's not then we likely raced with something else
-                // recycling this PID, so we just skip this step.
-                let mut res = 0;
-                let r = IsProcessInJob(p.inner, self.job.inner, &mut res);
-                if r == 0 {
-                    info!("failed to test is process in job: {}", last_err());
-                    return false
-                }
-                res == TRUE
-            });
-
+            let list = list.iter()
+                .filter(|&&id| {
+                    // let's not kill ourselves
+                    id as DWORD != GetCurrentProcessId()
+                })
+                .filter_map(|&id| {
+                    // Open the process with the necessary rights, and if this
+                    // fails then we probably raced with the process exiting so we
+                    // ignore the problem.
+                    let flags = PROCESS_QUERY_INFORMATION | PROCESS_TERMINATE | SYNCHRONIZE;
+                    let p = OpenProcess(flags, FALSE, id as DWORD);
+                    if p.is_null() {
+                        None
+                    } else {
+                        Some(Handle { inner: p })
+                    }
+                })
+                .filter(|p| {
+                    // Test if this process was actually in the job object or not.
+                    // If it's not then we likely raced with something else
+                    // recycling this PID, so we just skip this step.
+                    let mut res = 0;
+                    let r = IsProcessInJob(p.inner, self.job.inner, &mut res);
+                    if r == 0 {
+                        info!("failed to test is process in job: {}", last_err());
+                        return false;
+                    }
+                    res == TRUE
+                });
 
             for p in list {
                 // Load the file which this process was spawned from. We then
                 // later use this for identification purposes.
                 let mut buf = [0; 1024];
-                let r = GetProcessImageFileNameW(p.inner,
-                                                 buf.as_mut_ptr(),
-                                                 buf.len() as DWORD);
+                let r = GetProcessImageFileNameW(p.inner, buf.as_mut_ptr(), buf.len() as DWORD);
                 if r == 0 {
                     info!("failed to get image name: {}", last_err());
-                    continue
+                    continue;
                 }
                 let s = OsString::from_wide(&buf[..r as usize]);
                 info!("found remaining: {:?}", s);
@@ -235,7 +235,7 @@ mod imp {
                 if let Some(s) = s.to_str() {
                     if s.contains("mspdbsrv") {
                         info!("\toops, this is mspdbsrv");
-                        continue
+                        continue;
                     }
                 }
 
@@ -252,18 +252,20 @@ mod imp {
                 let r = WaitForSingleObject(p.inner, INFINITE);
                 if r != 0 {
                     info!("failed to wait for process to die: {}", last_err());
-                    return false
+                    return false;
                 }
                 killed = true;
             }
 
-            return killed
+            killed
         }
     }
 
     impl Drop for Handle {
         fn drop(&mut self) {
-            unsafe { CloseHandle(self.inner); }
+            unsafe {
+                CloseHandle(self.inner);
+            }
         }
     }
 }
