@@ -1,12 +1,14 @@
 use std::env;
 use std::ffi::{OsStr, OsString};
-use std::fs::File;
-use std::fs::OpenOptions;
+use std::fs::{self, File, OpenOptions};
+use std::io;
 use std::io::prelude::*;
-use std::path::{Path, PathBuf, Component};
+use std::path::{Component, Path, PathBuf};
+use std::iter;
 
-use util::{internal, CargoResult};
-use util::errors::{CargoResultExt, Internal, CargoError};
+use filetime::FileTime;
+
+use util::errors::{CargoError, CargoResult, CargoResultExt, Internal};
 
 pub fn join_paths<T: AsRef<OsStr>>(paths: &[T], env: &str) -> CargoResult<OsString> {
     let err = match env::join_paths(paths.iter()) {
@@ -17,16 +19,22 @@ pub fn join_paths<T: AsRef<OsStr>>(paths: &[T], env: &str) -> CargoResult<OsStri
     let err = CargoError::from(err);
     let explain = Internal::new(format_err!("failed to join path array: {:?}", paths));
     let err = CargoError::from(err.context(explain));
-    let more_explain = format!("failed to join search paths together\n\
-                                Does ${} have an unterminated quote character?",
-                               env);
-    return Err(err.context(more_explain).into())
+    let more_explain = format!(
+        "failed to join search paths together\n\
+         Does ${} have an unterminated quote character?",
+        env
+    );
+    Err(err.context(more_explain).into())
 }
 
 pub fn dylib_path_envvar() -> &'static str {
-    if cfg!(windows) {"PATH"}
-    else if cfg!(target_os = "macos") {"DYLD_LIBRARY_PATH"}
-    else {"LD_LIBRARY_PATH"}
+    if cfg!(windows) {
+        "PATH"
+    } else if cfg!(target_os = "macos") {
+        "DYLD_LIBRARY_PATH"
+    } else {
+        "LD_LIBRARY_PATH"
+    }
 }
 
 pub fn dylib_path() -> Vec<PathBuf> {
@@ -38,8 +46,7 @@ pub fn dylib_path() -> Vec<PathBuf> {
 
 pub fn normalize_path(path: &Path) -> PathBuf {
     let mut components = path.components().peekable();
-    let mut ret = if let Some(c @ Component::Prefix(..)) = components.peek()
-                                                                     .cloned() {
+    let mut ret = if let Some(c @ Component::Prefix(..)) = components.peek().cloned() {
         components.next();
         PathBuf::from(c.as_os_str())
     } else {
@@ -49,10 +56,16 @@ pub fn normalize_path(path: &Path) -> PathBuf {
     for component in components {
         match component {
             Component::Prefix(..) => unreachable!(),
-            Component::RootDir => { ret.push(component.as_os_str()); }
+            Component::RootDir => {
+                ret.push(component.as_os_str());
+            }
             Component::CurDir => {}
-            Component::ParentDir => { ret.pop(); }
-            Component::Normal(c) => { ret.push(c); }
+            Component::ParentDir => {
+                ret.pop();
+            }
+            Component::Normal(c) => {
+                ret.push(c);
+            }
         }
     }
     ret
@@ -72,6 +85,32 @@ pub fn without_prefix<'a>(long_path: &'a Path, prefix: &'a Path) -> Option<&'a P
     }
 }
 
+pub fn resolve_executable(exec: &Path) -> CargoResult<PathBuf> {
+    if exec.components().count() == 1 {
+        let paths = env::var_os("PATH").ok_or(format_err!("no PATH"))?;
+        let candidates = env::split_paths(&paths).flat_map(|path| {
+            let candidate = PathBuf::from(path).join(&exec);
+            let with_exe = if env::consts::EXE_EXTENSION == "" {
+                None
+            } else {
+                Some(candidate.with_extension(env::consts::EXE_EXTENSION))
+            };
+            iter::once(candidate).chain(with_exe)
+        });
+        for candidate in candidates {
+            if candidate.is_file() {
+                // PATH may have a component like "." in it, so we still need to
+                // canonicalize.
+                return Ok(candidate.canonicalize()?);
+            }
+        }
+
+        bail!("no executable for `{}` found in PATH", exec.display())
+    } else {
+        Ok(exec.canonicalize()?)
+    }
+}
+
 pub fn read(path: &Path) -> CargoResult<String> {
     match String::from_utf8(read_bytes(path)?) {
         Ok(s) => Ok(s),
@@ -88,9 +127,8 @@ pub fn read_bytes(path: &Path) -> CargoResult<Vec<u8>> {
         }
         f.read_to_end(&mut ret)?;
         Ok(ret)
-    })().chain_err(|| {
-        format!("failed to read `{}`", path.display())
-    })?;
+    })()
+        .chain_err(|| format!("failed to read `{}`", path.display()))?;
     Ok(res)
 }
 
@@ -99,28 +137,31 @@ pub fn write(path: &Path, contents: &[u8]) -> CargoResult<()> {
         let mut f = File::create(path)?;
         f.write_all(contents)?;
         Ok(())
-    })().chain_err(|| {
-        format!("failed to write `{}`", path.display())
-    })?;
+    })()
+        .chain_err(|| format!("failed to write `{}`", path.display()))?;
     Ok(())
 }
 
 pub fn append(path: &Path, contents: &[u8]) -> CargoResult<()> {
     (|| -> CargoResult<()> {
         let mut f = OpenOptions::new()
-                                 .write(true)
-                                 .append(true)
-                                 .create(true)
-                                 .open(path)?;
+            .write(true)
+            .append(true)
+            .create(true)
+            .open(path)?;
 
         f.write_all(contents)?;
         Ok(())
-    })().chain_err(|| {
-        internal(format!("failed to write `{}`", path.display()))
-    })?;
+    })()
+        .chain_err(|| format!("failed to write `{}`", path.display()))?;
     Ok(())
 }
 
+pub fn mtime(path: &Path) -> CargoResult<FileTime> {
+    let meta = fs::metadata(path).chain_err(|| format!("failed to stat `{}`", path.display()))?;
+    Ok(FileTime::from_last_modification_time(&meta))
+}
+
 #[cfg(unix)]
 pub fn path2bytes(path: &Path) -> CargoResult<&[u8]> {
     use std::os::unix::prelude::*;
@@ -130,8 +171,7 @@ pub fn path2bytes(path: &Path) -> CargoResult<&[u8]> {
 pub fn path2bytes(path: &Path) -> CargoResult<&[u8]> {
     match path.as_os_str().to_str() {
         Some(s) => Ok(s.as_bytes()),
-        None => Err(format_err!("invalid non-unicode path: {}",
-                                path.display())),
+        None => Err(format_err!("invalid non-unicode path: {}", path.display())),
     }
 }
 
@@ -156,7 +196,7 @@ pub fn ancestors(path: &Path) -> PathAncestors {
 
 pub struct PathAncestors<'a> {
     current: Option<&'a Path>,
-    stop_at: Option<PathBuf>
+    stop_at: Option<PathBuf>,
 }
 
 impl<'a> PathAncestors<'a> {
@@ -188,3 +228,65 @@ impl<'a> Iterator for PathAncestors<'a> {
         }
     }
 }
+
+pub fn remove_dir_all<P: AsRef<Path>>(p: P) -> CargoResult<()> {
+    _remove_dir_all(p.as_ref())
+}
+
+fn _remove_dir_all(p: &Path) -> CargoResult<()> {
+    if p.symlink_metadata()?.file_type().is_symlink() {
+        return remove_file(p);
+    }
+    let entries = p.read_dir()
+        .chain_err(|| format!("failed to read directory `{}`", p.display()))?;
+    for entry in entries {
+        let entry = entry?;
+        let path = entry.path();
+        if entry.file_type()?.is_dir() {
+            remove_dir_all(&path)?;
+        } else {
+            remove_file(&path)?;
+        }
+    }
+    remove_dir(&p)
+}
+
+pub fn remove_dir<P: AsRef<Path>>(p: P) -> CargoResult<()> {
+    _remove_dir(p.as_ref())
+}
+
+fn _remove_dir(p: &Path) -> CargoResult<()> {
+    fs::remove_dir(p).chain_err(|| format!("failed to remove directory `{}`", p.display()))?;
+    Ok(())
+}
+
+pub fn remove_file<P: AsRef<Path>>(p: P) -> CargoResult<()> {
+    _remove_file(p.as_ref())
+}
+
+fn _remove_file(p: &Path) -> CargoResult<()> {
+    let mut err = match fs::remove_file(p) {
+        Ok(()) => return Ok(()),
+        Err(e) => e,
+    };
+
+    if err.kind() == io::ErrorKind::PermissionDenied && set_not_readonly(p).unwrap_or(false) {
+        match fs::remove_file(p) {
+            Ok(()) => return Ok(()),
+            Err(e) => err = e,
+        }
+    }
+
+    Err(err).chain_err(|| format!("failed to remove file `{}`", p.display()))?;
+    Ok(())
+}
+
+fn set_not_readonly(p: &Path) -> io::Result<bool> {
+    let mut perms = p.metadata()?.permissions();
+    if !perms.readonly() {
+        return Ok(false);
+    }
+    perms.set_readonly(false);
+    fs::set_permissions(p, perms)?;
+    Ok(true)
+}
